#ifndef DICTBROWSER_H
#define DICTBROWSER_H

#include <QTextBrowser>
#include <QSyntaxHighlighter>


namespace Dict {

class Page;
class Hiliter;

class Browser : public QTextBrowser
{
	Q_OBJECT

public:
	Browser(QWidget *parent = 0);
	~Browser();
	QStringList backwardList(int len = 10) const
		{ return historyList(backwardHistory, len); }
	QStringList forwardList(int len = 10) const
		{ return historyList(forwardHistory, len); }
	int historySize() const { return histSize; }
	void setHistorySize(int size) { histSize = size; }
	QString pageTitle() const { return entry.title; }

	QFont textFont() const { return document()->defaultFont(); }
	void setTextFont(const QFont &font);
	void setStyleSheet(const QString &style);
	Page *dictPage() const { return page; }
	QString pageText() const;
	Hiliter *dictHiliter();
	void setHilite(bool on, QTextCharFormat);

public Q_SLOTS:
	void backward() { moveTo(-1); }
	void forward() { moveTo(1); }
	void moveTo(int steps);
	void setPage(const QString &query, const QString &text);
	void appendPage(const QString &query, const QString &text);
	void plainText(bool toggle);
	void clearHistory();
	void busyCursor(bool busy);
	void pointCursor(const QString &text);

	void savePage();
	void printPage();
	void previewPage();

private Q_SLOTS:
	void setPage(bool append);
	void selection();
	void print(QPrinter *printer) { QTextEdit::print(printer); }

Q_SIGNALS:
	void historyChanged(const QString &);

private:
	friend class Search;
	struct HistoryEntry {
		QString title, page;
		int hpos, vpos, type;
	};

	QList<HistoryEntry> backwardHistory, forwardHistory;
	HistoryEntry entry;
	QStringList historyList(const QList<HistoryEntry> &history, int len) const;
	int histSize;
	void savePosition();
	void restorePosition();

	bool plaintext;
	void displayPage(const QString &text);
	void reload();

	QCursor cursor;
	void clearSelection();

	void focusOutEvent(QFocusEvent *event);
	void contextMenuEvent(QContextMenuEvent *event);

	Page *page;
	Hiliter *hiliter;
};

}

#endif
