#include "widgets.h"


namespace Dict {

Database::Database(QObject *parent)
	: KActionMenu(parent), dbmenu(0), dbedit(0)
{
	setDelayed(false);
	connect(menu(), SIGNAL(triggered(QAction *)),
		SLOT(actionTriggered(QAction *)));

	group = new QActionGroup(this);
}

void Database::setMenu(Menu *menu, QAction *edit)
{
	dbmenu = menu;
	dbedit = edit;
	connect(dbmenu->menu(), SIGNAL(hovered(QAction *)), SLOT(update()));
}

void Database::setGroups(const GroupList &list)
{
	QString text;
	if (group->checkedAction())
		text = group->checkedAction()->text().remove('&');

	foreach (QAction *action, group->actions())
		group->removeAction(action);
	menu()->clear();

	addAction(i18n("First Match"), QStringList(Connection::DefaultDatabase));
	addAction(i18n("All Databases"), QStringList(Connection::AllDatabases));
	if (dbmenu) KActionMenu::addAction(dbmenu);

	addSeparator();
	foreach (GroupPair pair, list)
		addAction(pair.first, pair.second);

	foreach (QAction *action, menu()->actions()) {
		action->setCheckable(true);
		if (!action->isSeparator())
			group->addAction(action);
		if (action->text() == text)
			action->setChecked(true);
	}
	if (!group->checkedAction()) setChecked(0);
	addSeparator();
	if (dbedit) KActionMenu::addAction(dbedit);

	setEnabled(true);
}

void Database::update()
{
	dbmenu->setChecked(true);
	emit selected(dbmenu->current());
}

void Database::addAction(const QString &text, const QStringList &data)
{
	menu()->addAction(text)->setData(data);
}

void Database::actionTriggered(QAction *action)
{
	if (action->actionGroup() == group)
		emit selected(action->data().toStringList());
// 	else if (action != dbedit)
// 		emit selected(dbmenu->current());
}

void Database::setChecked(int select)
{
	if (select < 0 || select >= group->actions().size())
		select = 0;
	group->actions().at(select)->setChecked(true);
}

int Database::checked() const
{
	QAction *checked = group->checkedAction();
	return (checked) ? group->actions().indexOf(checked) : 0;
}

QStringList Database::current() const
{
	QAction *action = group->checkedAction();
	return (action == dbmenu)
		? dbmenu->current() : action->data().toStringList();
}

void Database::setList()
{
	dbmenu->setChecked(true);
}

void Database::checkList()
{
	QStringList dblist = dbmenu->list();
	dblist << Connection::DefaultDatabase << Connection::AllDatabases;
	foreach (QAction *action, group->actions()) {
		if (action == dbmenu) continue;
		QStringList list;
		foreach (QString db, action->data().toStringList())
			if (dblist.contains(db)) list << db;
		action->setData(list);
	}
}

}
