#include "widgets.h"


namespace Dict {

Menu::Menu(QObject *parent)
	: Info(parent), menuLen(24) {}

void Menu::setList(const List &list, const QStringList &selected)
{
	clear();

	QMenu *submenu = menu();
	for (int i = 0; i < list.size(); ++i) {
		if (menuLen > 0 && i && !(i % menuLen) && i != (list.size() - 1)) {
			KActionMenu *more = new KActionMenu(i18n("More Databases"), submenu);
			submenu->addSeparator();
			submenu->addAction(more);
			submenu = more->menu();
		}
		QAction *action = submenu->addAction(elide(list[i].second));
		action->setData(list[i].first);
		action->setCheckable(true);
	}
	setCurrent(selected);
	setEnabled(!list.isEmpty());
}

QStringList Menu::listData(const QMenu *submenu) const
{
	QStringList list;
	foreach (QAction *action, submenu->actions())
		if (action->isCheckable())
			list << action->data().toString();
		else if (action->menu())
			list += listData(action->menu());
	return list;
}

void Menu::setCurrentData(const QStringList &list, const QMenu *submenu)
{
	foreach (QAction *action, submenu->actions())
		if (action->isCheckable())
			action->setChecked(list.contains(action->data().toString()));
		else if (action->menu())
			setCurrentData(list, action->menu());
}

QStringList Menu::currentData(const QMenu *submenu) const
{
	QStringList list;
	foreach (QAction *action, submenu->actions())
		if (action->isChecked())
			list << action->data().toString();
		else if (action->menu())
			list += currentData(action->menu());
	return list;
}

void Menu::actionTriggered(QAction *action)
{
	emit selected(current());
}

}
