#include "widgets.h"


namespace Dict {

View::View(QWidget *parent)
	: QListWidget(parent)
{
	setSelectionMode(QAbstractItemView::MultiSelection);
	connect(this, SIGNAL(itemSelectionChanged()), SLOT(selection()));
}

void View::selection()
{
	QStringList list;
	for (int i = 0; i < count(); ++i)
		if (item(i)->isSelected())
			list << item(i)->data(Qt::UserRole).toString();
	emit selected(list);
}

void View::setList(const List &list, const QStringList &selected)
{
	blockSignals(true);
	clear();
	blockSignals(false);
	foreach (Pair pair, list)
		addItem(elide(pair.second));
	for (int i = 0; i < count(); ++i)
		item(i)->setData(Qt::UserRole, list[i].first);
	setCurrent(selected);
}

QString View::data(int index) const
{
	return item(index)->data(Qt::UserRole).toString();
}

void View::setCurrent(const QStringList &list)
{
	blockSignals(true);
	for (int i = 0; i < count(); ++i)
		item(i)->setSelected(list.contains(data(i)));
	blockSignals(false);
}

const QList<QListWidgetItem *> View::list() const
{
	QList<QListWidgetItem *> itemlist;
	for (int i = 0; i < count(); ++i)
		itemlist << item(i);
	return itemlist;
}

}
