#ifndef DICTWIDGETS_H
#define DICTWIDGETS_H

#include "../browser.h"
#include "../hiliter.h"
#include "../group.h"
#include "../net/dict.h"
#include "ui_searchbar.h"

#include <KActionMenu>
#include <KSelectAction>
#include <KMenu>

#include <QWidget>
#include <QComboBox>
#include <QListWidget>
#include <QProgressBar>
#include <QTimer>


namespace Dict {

QString elide(const QString &text, int len = 60);

class Info : public KActionMenu
{
	Q_OBJECT

public:
	Info(QObject *parent = 0);
	void addAction(const QString &text, const QString &data);
	QStringList list() const;

Q_SIGNALS:
	void selected(const QString &);

protected Q_SLOTS:
	virtual void actionTriggered(QAction *action);
};


class Select : public KSelectAction
{
	Q_OBJECT

public:
	Select(QObject *parent = 0);
	QString current() const;
	void setList(const List &list, const QString &selected);
	void setEnabled(bool);
	void clear();

public Q_SLOTS:
	void setCurrent(const QString &data);
	void setFocus();

private Q_SLOTS:
	void actionTriggered(QAction *action);

private:
	QWidget *createWidget(QWidget *parent);
	void deleteWidget(QWidget *parent);

	QComboBox *combobox;
};


class Menu : public Info
{
	Q_OBJECT

public:
	Menu(QObject *parent = 0);
	QStringList current() const { return currentData(QAction::menu()); }
	void setList(const List &list, const QStringList &selected);
	QStringList list() const { return listData(QAction::menu()); }
	void clear() { menu()->clear(); }
	void setMenuLength(int length) { menuLen = length; }

public Q_SLOTS:
	void setCurrent(const QStringList &data) { setCurrentData(data, menu()); }

Q_SIGNALS:
	void selected(const QStringList &);

private Q_SLOTS:
	void actionTriggered(QAction *action);

private:
	QStringList listData(const QMenu *submenu) const;
	QStringList currentData(const QMenu *submenu) const;
	void setCurrentData(const QStringList &data, const QMenu *submenu);

	int menuLen;
};


class View : public QListWidget
{
	Q_OBJECT

public:
	View(QWidget *parent = 0);
	void setList(const List &list, const QStringList &selected);
	const QList<QListWidgetItem *> list() const;

Q_SIGNALS:
	void selected(const QStringList&);

public Q_SLOTS:
	void setCurrent(const QStringList &list);

private Q_SLOTS:
	void selection();

private:
	QString data(int index) const;
};


class Database : public KActionMenu
{
	Q_OBJECT

public:
	Database(QObject *parent = 0);
	void setMenu(Menu *menu, QAction *edit);
	void setGroups(const GroupList &list);
	void checkList(void);
	void setChecked(int);
	int checked() const;
	QStringList current() const;

Q_SIGNALS:
	void selected(const QStringList &);

public Q_SLOTS:
	void setList();

private Q_SLOTS:
	void update();
	void actionTriggered(QAction *action);

private:
	void addAction(const QString &text, const QStringList &data);
	QActionGroup *group;
	Menu *dbmenu;
	QAction *dbedit;
};


class HistButton;

class History : public KAction
{
	Q_OBJECT

public:
	History(Browser *parent);
	QWidget *createWidget(QWidget *parent);
	void deleteWidget(QWidget *widget);

public Q_SLOTS:
	void setEnabled(bool enabled = true);

Q_SIGNALS:
	void activated();

private Q_SLOTS:
	void move(QAction *action);
	void set();

private:
	Browser *browser;
	HistButton *button;
};


class Search : public QWidget, public Ui::Searchbar
{
	Q_OBJECT

public:
	Search(Browser *parent);

public Q_SLOTS:
	void next() { find(); }
	void prev() { find(QTextDocument::FindBackward); }

	void show();
	void hide();
	void clear();

private Q_SLOTS:
	void find(QTextDocument::FindFlags direction = 0);
	void search(const QString &text);
	void hiliteAll(bool hilite);

private:
	Browser *browser;
	bool found;

	void keyPressEvent(QKeyEvent *event);
};


class Status : public QWidget
{
	Q_OBJECT

public:
	Status(QWidget *parent);

public Q_SLOTS:
	void setStatus(const char *icon, const QString &text, bool enabled);
	void setTip(const QString &text);

private:
	QLabel *picture, *label;
};


class Progress : public QProgressBar
{
	Q_OBJECT

public:
	Progress(QWidget *parent);

public Q_SLOTS:
	void set(int current, int total);

private:
	QTimer timer;
};


class Entry : public QComboBox
{
	Q_OBJECT

public:
	Entry(QWidget *parent);

public Q_SLOTS:
	void define();
	void match();

Q_SIGNALS:
	void matchText(const QString &);
};

}

#endif
