#ifndef DICTPAGE_H
#define DICTPAGE_H

#include "formatter.h"
#include "net/dict.h"


namespace Dict {

class Page : public QObject
{
	Q_OBJECT

public:
	enum Warning {
		LongSelection = 1, UrlSelection, UrlRequested
	};
	enum Error {
		UnknownDatabase = 1, MalformedURL, UnsupportedScheme,
		FileSystem
	};
	Page() { headings = true; correct = false; reformat = true; }
	Page(const Response &response);
	void set(const Response &response, bool append = false);
	void setTitle(const QString &title) { query = title; }
	QString title() const { return query; }
	QString htmltext() const { return html; }

	void error(const QString &text, int code);
	void warning(const QString &text, Warning type);
	void about(const QString &text, const QString &version);
	void help(const QString &text);
	void nomatches(const QString &text, bool define);
	void corrections(bool found);
	void correcting(bool on) { correct = on; }
	void status(const Connection *dict);

	int type() const { return typeCode; }
	void setHeadings(bool on) { headings = on; }
	void setReformat(bool on) { reformat = on; }
	Formatter *dictFormatter() { return &formatter; }

	void setMaxMatches(int limit) { maxMatches = limit; }
	void setMaxSuggestions(int limit) { maxSuggestions = limit; }

Q_SIGNALS:
	void changed(bool append = false);

private:
	bool headings, correct, reformat;
	static int maxMatches, maxSuggestions;
	QString query, html;
	int typeCode;
	Formatter formatter;

	void heading(const QString &text);
	void matches(const QStringList &list);
	void server(const QString &text);
	void databases(const List &list);
	void strategies(const List &list);
	void suggestions(const QString &text, const QStringList &list);
	void status(const QString &message);
	void status(const QString &title, const QString &desc);
	void title(const QString &title);
};

}

#endif
