#include "selection.h"

#include <KModifierKeyInfo>
#include <QApplication>
#include <QClipboard>


namespace Dict {

Selection::Selection()
{
	clipboard = QApplication::clipboard();
	select.yank = false;
	modifier = (Qt::Key)0;
	modinfo = 0;
}

void Selection::setup(const SelectionConf &conf)
{
	set(conf.yank);
	select = conf;
	switch (conf.modifier) {
	case None: default: modifier = (Qt::Key)0; break;
	case Ctrl: modifier = Qt::Key_Control; break;
	case Alt: modifier = Qt::Key_Alt; break;
	case Meta: modifier = Qt::Key_Meta; break;
	case Shift: modifier = Qt::Key_Shift; break;
	}
	if (modifier && !modinfo)
		modinfo = new KModifierKeyInfo(this);
}

void Selection::set(bool yank)
{
	if (select.yank == yank)
		return;
	else
		select.yank = yank;

	if (yank)
		connect(clipboard, SIGNAL(selectionChanged()), SLOT(selection()));
	else
		clipboard->disconnect(this);

#ifndef Q_WS_X11
	if (yank)
		setTimeout(1000);
	else
		setTimeout(0);
#endif
}

static QString last;

QString Selection::current()
{
	return clipboard->text(QClipboard::Selection).left(select.maxLen);
}

void Selection::selection()
{
	last = current();

	if (modifier && !modinfo->isKeyPressed(modifier))
		return;

	bool owner = clipboard->ownsSelection();
	if (!select.local && owner)
		return;
	if (!select.remote && !owner)
		return;

	if (last.length() < select.minLen || last.length() >= select.maxLen)
		return;

	emit selectionChanged(last);
}

#ifndef Q_WS_X11

void Selection::setTimeout(int timeout)
{
	if (timer)
		killTimer(timer);
	if (timeout) {
		last = current();
		timer = startTimer(timeout);
	}
}

void Selection::timerEvent(QTimerEvent *event)
{
	if (last != current()) {
		last = current();
		if (last.length() >= select.minLen && last.length() < select.maxLen)
			emit selectionChanged(last);
	}
}

#endif

}
