#ifndef DICTSETTINGS_H
#define DICTSETTINGS_H

#include "ui_settings.h"


namespace Dict {

class SelectionConf;

class Conf
{
	friend class Settings;
	friend class Window;
	KSharedConfigPtr config;
	QString dicthost;

	enum UserAgent {
		Short, Long, Custom
	};
	UserAgent agent;
	QString client;

	bool corrections, headings, matchPage, autoMatch, reformat, hilite, filter;
	int sidebarSide, defaultSearch, autoStrategy, maxMatches, maxSuggestions;

public:
	Conf();
	~Conf();
	void setHost(const QString &host) { dicthost = host; }

	void read();
	QStringList server();
	QString host();
	int port();
	QStringList database();
	QString strategy();
	int selected();

	void write();
	void writeServer(const QStringList &hosts);
	void writeHost(const QStringList &database, const QString &strategy);
	void writeSelected(int selected);

	bool optMime();
	QString username();
	QString password();
	void writeUser(const QString &username, const QString &password);
	void deleteUser();

	int sidebar();
	QList<int> widths();
	int stack();
	bool splitter();

	void writeSidebar(int index);
	void writeWidths(QList<int> sizes);

	QFont font();
	void writeFont(const QFont &font);

	SelectionConf selection(void);
	void writeSelection(const SelectionConf &conf);

	QStringList groups();
	void writeGroups(const QStringList &groups);

	QString userAgent() const;
	QString userAgent(UserAgent) const;
};

class Style;

class Settings : public KDialog, public Ui::Settings
{
public:
	Settings(QWidget *parent);

	void setHosts(const QStringList &hosts);
	QStringList hosts();

	void setFont(const QFont &font);
	QFont font();

	void setStyle(Style *style);

	void setSelection(const SelectionConf &conf);
	SelectionConf selection(void);

	void readConf(const Conf &conf);
	void writeConf(Conf &conf);
};

}

#endif
