#include "tracker.h"
#include "page.h"


namespace Dict {

Tracker::Tracker(Connection *d, Page *w, Page *m)
	: dict(d), wordPage(w), matchPage(m)
{
	correct = false;
	filter = false;
	matchlist = false;
	dictinit = Stopped;
	setDefaultDatabase(0);
	clear();
}

void Tracker::define(const QString &text)
{
	if (!(dictinit & Started)) init();
	query = text;
	reset();
	defining = true;
	dbcount = database.size();
	emit loading(true);

	if (dbcount)
		foreach (QString dbase, database)
			dict->define(query, dbase);
	else {
		dbcount = 1;
		dict->define(query, defaultDatabase);
	}
	emit progress(0, dbcount);
}

void Tracker::match(const QString &text, const QString &strategy)
{
	if (!(dictinit & Started)) init();
	query = text;
	matching = true;
	dbcount = database.size();
	QString strat = (!strategy.isEmpty()) ? strategy : this->strategy;
	emit loading(true);

	if (dbcount)
		foreach (QString dbase, database)
			dict->match(query, dbase, strat);
	else {
		dbcount = 1;
		dict->match(query, defaultDatabase, strat);
	}
	emit progress(0, dbcount);
}

void Tracker::info(const QString &text, const QString &info)
{
	if (!(dictinit & Started)) init();
	query = text;
	emit loading(true);

	if (info == "status")
		dict->status();
	else
		dict->info(info);
	emit progress(0, 1);
}

void Tracker::cancel()
{
	dict->cancel();
	if (dictinit == Started)
		dictinit = Stopped;
	clear();
}

void Tracker::clear()
{
	reset();
	defining = matching = correcting = appending = automatching = false;
	emit loading(false);
}

void Tracker::setDatabase(const QStringList &list)
{
	database = list;
}

void Tracker::setStrategy(const QString &text)
{
	strategy = (!text.isEmpty()) ? text : Connection::DefaultStrategy;
	if (autoStrategy != Connection::DefaultStrategy)
		autoStrategy = strategy;
}

void Tracker::setDefaultDatabase(int dbase)
{
	defaultDatabase = (dbase) ? Connection::AllDatabases : Connection::DefaultDatabase;
}

void Tracker::setAutoStrategy(int strat)
{
	autoStrategy = (strat) ? strategy : Connection::DefaultStrategy;
}

void Tracker::init()
{
	dictinit = Started;
	dict->info("databases");
	dict->info("strategies");
// 	emit loading(true);
}

void Tracker::done()
{
	if (automatch && matchPage && !automatching && defining
		&& (filter || !matchlist))
		automatches();
	else
		clear();
	matchlist = false;
}

void Tracker::suggestions()
{
	reset();
	correcting = true;
	match(query, Connection::DefaultStrategy);
}

void Tracker::automatches()
{
	reset();
	automatching = true;
	appending = false;
	correcting = false;
	match(query, autoStrategy);
}

void Tracker::response(const Response &response)
{
	Page *page = wordPage;

	switch (response.type) {
	case Response::Words:
		emit words();
		break;
	case Response::Matches:
		page->correcting(correcting);
		if (correcting)
			emit corrections();
		else if (matchPage) {
			page = matchPage;
			emit matches(!automatching);
		}
		break;
	case Response::NoMatch:
		++misses;
		emit progress(count(), dbcount);
		if (misses < dbcount) {
			if (count() >= dbcount)
				done();
			return;
		} else if (automatching) {
		// do nothing
		} else if (!correcting) {
			page->nomatches(query, defining);
			if (correct) {
				suggestions();
				return;
			}
		} else
			page->corrections(false);
		break;
	case Response::DatabaseList:
		emit databases(((ListResponse &)response).list);
		if (!(dictinit & Databases)) {
			dictinit |= Databases;
			return;
		}
		break;
	case Response::StrategyList:
		emit strategies(((ListResponse &)response).list);
		if (!(dictinit & Strategies)) {
// 			emit loading(false);
			dictinit |= Strategies;
			return;
		}
		break;
	case Response::ServerInfo:
	case Response::DatabaseInfo:
		break;
	case Response::StatusMessage:
		page->status(dict);
		break;
	case Response::AuthOK:
		init();
		break;
	default:
		qWarning() << "dict tracker: unhandled type" << response.type;
	}

	page->setTitle(query);
	page->set(response, appending);
	appending = true;

	if (response.type == Response::Words
	|| response.type == Response::Matches) {
		if (response.done()) ++hits;
		emit progress(count(), dbcount);
		if (count() >= dbcount)
			done();
	} else {
		if (response.type != Response::NoMatch)
			emit progress(1, 1);
		done();
	}
}

}
