#ifndef DICTTRACKER_H
#define DICTTRACKER_H

#include "net/dict.h"


namespace Dict {

class Page;

class Tracker : public QObject
{
	Q_OBJECT

public:
	Tracker(Connection *dict, Page *word, Page *match = 0);

	void define(const QString &query);
	void match(const QString &query, const QString &strategy);
	void info(const QString &query, const QString &text);
	void init();
	void cancel();

	void setCorrect(bool on) { correct = on; }
	void setMatchPage(Page *match) { matchPage = match; }
	void setAutoMatch(bool on) { automatch = on; }
	void setAutoStrategy(int strategy);
	void setDefaultDatabase(int database);
	void setMatchFilter(bool on) { filter = on; }
	void setMatchList(bool list) { matchlist = list; }

	QStringList databases() const { return database; }

private:
	enum Init { Stopped, Started, Databases, Strategies = 4 };
	int dictinit;
	bool defining, matching, correcting, appending, automatching;
	int hits, misses, dbcount;
	void reset() { hits = misses = 0; }
	int count() { return hits + misses; }
	void clear();
	void done();
	void suggestions();
	void automatches();

	Connection *dict;
	Page *wordPage, *matchPage;
	bool correct, automatch;
	bool filter, matchlist;
	QString query, strategy;
	QStringList database;
	QString defaultDatabase, defaultStrategy, autoStrategy;

public Q_SLOTS:
	void response(const Response &response);
	void setDatabase(const QStringList &database);
	void setStrategy(const QString &strategy);

Q_SIGNALS:
	void words();
	void matches(bool);
	void corrections();
	void loading(bool);
	void databases(const List &list);
	void strategies(const List &list);
	void progress(int current, int total);
};

}

#endif
