#ifndef DICTWINDOW_H
#define DICTWINDOW_H

#include "settings.h"
#include "gui/widgets.h"
#include "net/dict.h"

#include <KXmlGuiWindow>
#include <KUniqueApplication>

class QSplitter;
class QStackedWidget;
class QComboBox;
class QLabel;
class KSelectAction;


class Dikt : public KUniqueApplication
{
public:
	static const char *AppName;
	static const char *Version;
	int newInstance();
};


namespace Dict {

class Browser;
class Page;
class Tracker;
class Selection;
class Style;
class Settings;


class Window : public KXmlGuiWindow
{
	Q_OBJECT

public:
	Window();
	~Window();
	void startup(const QString &location = "");
	QSize sizeHint() const { return QSize(600, 500); }

public Q_SLOTS:
	void define(const QString &query);
	void match(const QString &query);

protected Q_SLOTS:
	void define(const QUrl &link);
	void setHost(const QString &host);
	void setDatabaseMenu(const List &list);
	void setStrategyMenu(const List &list);
	void setQuery(const QString &text) { query = text; }

	void configure();
	void authenticate();
	void intro() { about("about:"); }
	void about(const QString &query);
	void info(const QString &text) { info("about:" + text, text); }
	void info(const QString &query, const QString &text);
	void error(const QString &text, int code);
	void warning(const QString &text, int code);
	void cancel();
	void help() { about("about:help"); }

	void newWindow();
	void setHilite(bool on);
	void reformat();

private:
	static const int maxQueryLen = 120;
	QComboBox *entry;
	Browser *browser, *matchList;
	Info *dictinfo;
	KSelectAction *hostMenu;
	Select *strategyMenu;
	Menu *databaseMenu;
	View *databaseView;
	Connection *dict;
	Database *dbmenu;

	Settings *settings;
	QString query;
	Search *search;
	QSplitter *splitter;
	QStackedWidget *sidebar;
	KAction *stop, *yank, *auth, *edit, *hilite;
	Status *hostStatus, *userStatus;
	Conf conf;
	Page *page;
	Tracker *tracker;
	Selection *selection;
	Style *style;
	Progress *progress;

	void setupActions();
	void setupBrowser();
	void setupDict();
	void setupTracker();
	void setupStatusbar();
	void setupMenubar();
	void dicturl(const QUrl &url);

	QString dicthost() { return hostMenu->currentText(); }
	QStringList database() { return databaseMenu->current(); }
	QString strategy() { return strategyMenu->current(); }

private Q_SLOTS:
	void readConfig();
	void writeConfig();
	void hostConfig(const QString &host);
	void serverConfig();
	void loadSettings();
	void loadConfig();
	void setHostStatus(bool connected);
	void setUserStatus(bool authenticated);
	void setCapabilities(bool identified);
	void setGroups();

	void statusMessage(const QString &message);
	void toggleMenubar();
	void setSidebar();
	void setSidebar(int side);
	void setSidebar(QWidget *widget);
	void toggleDatabaseList();
	void toggleMatchList();
	void openMatchList(bool focus = false);
	void enableMatchList(bool enable);
	void nextFrame();
	void yankClipboard(const QString &selection);
};

}

#endif
